#!/usr/bin/env python3


import socket

# pylint: disable=import-error
from cmk.utils.site import url_prefix
from cmk.gui.i18n import _
from cmk.gui.valuespec import (
    Dictionary,
    TextAscii,
    CascadingDropdown,
    DropdownChoice,
    FixedValue,
    DEF_VALUE,
    TextInput,
    Transform,
    TextAreaUnicode,
)

from cmk.gui.plugins.wato import (
    notification_parameter_registry,
    NotificationParameter,
    HTTPProxyReference,
)

from cmk.gui.watolib.password_store import passwordstore_choices


# now way to import helper functions, giving up....
def local_site_url() -> str:
    return "https://" + socket.gethostname() + url_prefix() + "check_mk/"


def get_url_prefix_specs(default_choice, default_value=DEF_VALUE):
    return Transform(
        valuespec=CascadingDropdown(
            title=_("Include link to the item."),
            help=_(
                "If set, you use <b>Automatic HTTP(S)</b>, the URL prefix for host "
                "and service links within the notification is filled "
                "automatically. If you specify an URL prefix here, then "
                "several parts of the notification are armed with hyperlinks "
                "to your Checkmk GUI. In both cases, the recipient of the "
                "notification can directly visit the host or service in "
                "question in Checkmk. Specify an absolute URL including the "
                "<tt>.../check_mk/</tt>."
            ),
            choices=[
                ("automatic_http", _("Automatic HTTP link")),
                ("automatic_https", _("Automatic HTTPs link")),
                (
                    "manual",
                    _("Specify URL prefix"),
                    TextInput(
                        regex="^(http|https)://.*/check_mk/$",
                        regex_error=_(
                            "The URL must begin with <tt>http</tt> or "
                            "<tt>https</tt> and end with <tt>/check_mk/</tt>."
                            "regex: '^(http|https)://.*/check_mk/$'"
                        ),
                        size=64,
                        default_value=default_choice,
                    ),
                ),
            ],
            default_value=default_value,
        ),
        to_valuespec=_transform_to_valuespec_html_mail_url_prefix,
        from_valuespec=_transform_from_valuespec_html_mail_url_prefix,
    )


# We have to transform because 'add_to_event_context'
# in modules/events.py can't handle complex data structures
def _transform_from_valuespec_html_mail_url_prefix(p):  # pylint: disable=invalid-name
    if isinstance(p, tuple):
        return {p[0]: p[1]}
    if p == "automatic_http":
        return {"automatic": "http"}
    if p == "automatic_https":
        return {"automatic": "https"}
    return {"manual": p}


def _transform_to_valuespec_html_mail_url_prefix(p):  # pylint: disable=invalid-name,inconsistent-return-statements
    if not isinstance(p, dict):
        return ("manual", p)

    k, v = list(p.items())[0]  # pylint: disable=invalid-name
    if k == "automatic":
        return f"{k}_{v}"


@notification_parameter_registry.register
class NotificationParameterNtfy(NotificationParameter):
    @property
    def ident(self):
        return "ntfy"

    @property
    def spec(self) -> Dictionary:
        return Dictionary(
            title=_("Create notification with the following parameters"),
            optional_keys=[
                "ntfy_instance",
                "ntfy_access_token",
                "ntfy_include_cmk_icon",
                "ntfy_host_msg_tmpl",
                "ntfy_svc_msg_tmpl",
                "url_prefix",
                "ignore_ssl",
                "proxy_url",
            ],
            elements=[
                (
                    "ntfy_topic",
                    TextAscii(
                        title=_("Topic"),
                        help=_("Topic on the instance we should post notifications to"),
                        size=64,
                        allow_empty=False,
                    ),
                ),
                (
                    "ntfy_instance",
                    TextAscii(
                        title=_("ntfy Instance."),
                        help=_("Change value if you run your own instance"),
                        default_value="ntfy.sh",
                        size=64,
                    ),
                ),
                (
                    "ntfy_access_token",
                    CascadingDropdown(
                        title=_("ntfy access token"),
                        help=_(
                            "If an access token is needed to post to the channel, specify here. Username:Password Authentication is *not* supported (see: https://docs.ntfy.sh/config/#access-tokens)"
                        ),
                        choices=[
                            (
                                "ntfy_access_token",
                                _("Access token"),
                                TextAscii(size=80, allow_empty=False),
                            ),
                            (
                                "store",
                                _("Access token from password store"),
                                DropdownChoice(sorted=True, choices=passwordstore_choices),
                            ),
                        ],
                    ),
                ),
                (
                    "ntfy_include_cmk_icon",
                    FixedValue(
                        value=True,
                        title=_("Include Check_mk icon in Notification Message"),
                        totext=_("Include Check_mk icon in Notification Message"),
                        help=_("Might be usefull if notifications are not send to an dedicated topic."),
                    ),
                ),
                (
                    "ntfy_host_msg_tmpl",
                    TextAreaUnicode(
                        title=_("Service notification template."),
                        help=_(
                            "Override default notification template. Note that first empty line seperates message title from body"
                        ),
                        rows=9,
                        cols=58,
                        monospaced=True,
                        default_value="Host: $HOSTALIAS$ $HOSTSTATE$\n\n$HOSTOUTPUT$",
                    ),
                ),
                (
                    "ntfy_svc_msg_tmpl",
                    TextAreaUnicode(
                        title=_("Service notification template."),
                        help=_(
                            "Override default notification template. Note that first empty line seperates message title from body"
                        ),
                        rows=9,
                        cols=58,
                        monospaced=True,
                        default_value="Service: $HOSTALIAS$ / $SERVICEDESC$ $SERVICESTATE$\n\n$SERVICEOUTPUT$",
                    ),
                ),
                ("url_prefix", get_url_prefix_specs(local_site_url)),
                (
                    "ignore_ssl",
                    FixedValue(
                        value=True,
                        title=_("Disable SSL certificate verification"),
                        totext=_("Disable SSL certificate verification"),
                        help=_("Ignore unverified HTTPS request warnings. Use with caution."),
                    ),
                ),
                ("proxy_url", HTTPProxyReference()),
            ],
        )
